#!/usr/bin/perl -w

###########################################
# Program for drawing MRTG summary graphs # 
###########################################

use strict;
use locale;
use CGI qw(:standard);
use lib qw(/usr/local/rrdtool/lib/perl); # RRDTool PERL libraries path
use RRDs;

my $datapath="/usr/local/mrtg/data"; # RRD files path
my $version='1.2';

if (!param()) { die "ERROR: Program called without parameters.\n"; }

my $name; # Requested graph name
if (!(param("name"))) {
	$name="No_name";
}
else {
	$name=param("name"); # Processing of name variable
	if ($name =~ /^(\w+)$/) {
		$name = $1;
	}
	else { die "ERROR: Bad parameter name=$name\n"; }
}

my $log; # targets separated by spaces
if (!(param("log"))) { die "ERROR: Parameter log not defined.\n"; }
else { # Testing all targets
	$log=param("log");
	if ($log =~ /\//) { die "ERROR: Bad parameter log=$log\n"; }
	my $i;
	my $rrdinfo;
	foreach $i (split(" ",$log)) {
		$rrdinfo = RRDs::info "$datapath/$i.rrd";
		if (RRDs::error) {
			my $rrd_error = RRDs::error;
			die "RRDTOOL ERROR: $rrd_error\n";
		}
	}
}
# Graph type parameter
my $type; # PNG name variable
if (defined(param("type"))) {
	$type = param("type");
	chomp $type;
	if ($type =~ /^(\w+)$/) {
		$type = $1;
	}
	else { die "ERROR: Bad parameter type=$type\n"; }
	if ($type eq "normal") {
	}
	elsif ($type eq "neg") {
	}
	else { die "ERROR: Unknown parameter type=$type\n"; }
}
else {
	$type = "normal";
}

# HTTP and HTML page print
if (!param("png")) { # Only when variable png is not given
	print "Content-Type: text/html\n\n";
	print <<EOF
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
	"http://www.w3.org/TR/2000/REC-xhtml1-20000126/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-2" />
<link rel="stylesheet" type="text/css" href="../css/mrtg.css" />
<link rel="shortcut icon" type="image/x-icon" href="../favicon.ico" />
<title>MRTGsum</title>
</head>
<body>
EOF
;
	my $rrdfile; # Name of RRD file
	my $outrrds=""; # List of RRD files for printing
	my $router; # Separated router name
	my $target; # Separated target
	my $localtime = localtime(time); # Getting actual time
	foreach $rrdfile (split(" ",$log)) {
		$router = $rrdfile;
		$router =~ (s/^(\S+?)_.*$/$1/);
		$target = $rrdfile;
		$outrrds = $outrrds.' + '."<a href=\"14all.cgi?log=$target&amp;cfg=$router.cfg\">$target<\/a>";
	}
	$outrrds =~ (s/^ \+ //);
	print "<h1>Traffic summary graphs for $name<\/h1>\n";
	print "<div>$outrrds<\/div>\n";
	print "<hr />\n";
	print "<div>The statistics were last updated: <b>$localtime<\/b><\/div>\n";
	print "<hr />\n";
	print "<h2>\'Daily\' graph \(5 Minute Average\)<\/h2>\n";
	print "<div><img alt=\"daily-graph\" src=\"mrtgsum.cgi?name=$name&amp;log=$log&amp;png=daily&amp;type=$type\" /><\/div>\n";
	print "<h2>\'Weekly\' graph \(30 Minute Average\)<\/h2>\n";
	print "<div><img alt=\"weekly-graph\" src=\"mrtgsum.cgi?name=$name&amp;log=$log&amp;png=weekly&amp;type=$type\" /><\/div>\n";
	print "<h2>\'Monthly\' graph \(2 Hour Average\)<\/h2>\n";
	print "<div><img alt=\"monthly-graph\" src=\"mrtgsum.cgi?name=$name&amp;log=$log&amp;png=monthly&amp;type=$type\" /><\/div>\n";
	print "<h2>\'Yearly\' graph \(1 Day Average\)<\/h2>\n";
	print "<div><img alt=\"yearly-graph\" src=\"mrtgsum.cgi?name=$name&amp;log=$log&amp;png=yearly&amp;type=$type\" /><\/div>\n";
	print "<div class=\"copyright\">\n";
	print "mrtgsum.cgi $version &copy;2004 <a href=\"http:\/\/www.pavouk.org\">Pavouk<\/a>\n";
	print "<\/div>\n";
	print "<\/body>\n";
	print "<\/html>\n";
}

# Routine for PNG printing
if (defined(param("png"))) {
	my $start; # Graph start time
	my $png; # PNG name variable
	$png=param("png");
	chomp $png;
	if ($png =~ /^(\w+)$/) {
		$png = $1;
	}
	else { die "ERROR: Bad parameter png=$png\n"; }
	if ($png eq "daily") {
		$start = '-2000m';
	}
	elsif ($png eq "weekly") {
		$start = '-12000m';
	}
	elsif ($png eq "monthly") {
		$start = '-800h';
	}
	elsif ($png eq "yearly") {
		$start = '-400d';
	}
	else {
		die "ERROR: Unknown parameter png=$png\n";
	}
	my @args; # RRDTool parameters array
	my @argsin; # temporary INPUT parameters for RRDTool
	my @argsout; # temporary OUTPUT parameters for RRDTool
	push @args, "/tmp/$name\-$png.png", # name of output file
	'--start', "$start", # start time
	'--end', 'now', # end time
	'--imgformat', 'PNG', # 
	'--base', '1000',
	'--vertical-label', '< OUT bps IN >',
	'--alt-y-mrtg',
	'--color', 'MGRID#ee0000',
	'--color', 'GRID#000000';
	if ($type eq "normal") {
		push @args,'--width', '400', '--height', '100';
		push @args, "COMMENT:\"$name\"\\c";
	}
	if ($type eq "neg") {
		push @args,'--width', '400', '--height', '200';
		push @args, "COMMENT:\"$name\"                   Maximal     Average     Current\\r";
	}
	my $i; # Get the name of target
	my $idef=""; # Name of def
	my $indefs=""; # Average input defs
	my $outdefs=""; # Average output defs
	my $mindefs=""; # Max input defs
	my $moutdefs=""; # Max output defs
	my $math; # Positives for CDEF
	my $defcount=0; # DEF counter
	my $rrdinfo; # RRD Info variable
	my $maxbytes=0; # Maxbytes is summary of all targets from RRD Info
	# Colors for INPUT values
	my @colorsin = (
		'30FC30',
		'3096FC',
		'FFC930',
		'FF3030',
		'30CC30',
		'3030CC',
		'FFFF00',
		'CC3030'
	);
	# Colors for OUTPUT values
	my @colorsout = (
		'00CC00',
		'0066CC',
		'FF9900',
		'CC0000',
		'009900',
		'000099',
		'CCCC00',
		'990000'
	);
	foreach $i (split(" ",$log)) { # Filling of array @args with defs
		# Computing maxbytes
		$rrdinfo = RRDs::info "$datapath/$i.rrd";
		if (RRDs::error) {
			my $rrd_error = RRDs::error;
			die "RRDTOOL ERROR: $rrd_error\n";
		}
		$maxbytes = $maxbytes + $$rrdinfo{"ds[ds0].max"};
		# Get colors
		my $colorin = shift @colorsin;
		push @colorsin, $colorin;
		my $colorout = shift @colorsout;
		push @colorsout, $colorout;
		# Getting RRD data
		$defcount++;
		$idef = 'def'.$defcount;
		push @args, "DEF:in$idef=$datapath\/$i.rrd:ds0:AVERAGE";
		push @args, "DEF:min$idef=$datapath\/$i.rrd:ds0:MAX";
		push @args, "DEF:out$idef=$datapath\/$i.rrd:ds1:AVERAGE";
		push @args, "DEF:mout$idef=$datapath\/$i.rrd:ds1:MAX";
		if ($type eq "neg") {
			push @args, "CDEF:compin$idef=in$idef,8,*";
			push @args, "CDEF:compmin$idef=min$idef,8,*";
			push @args, "CDEF:compout$idef=out$idef,8,*,-1,*";
			push @args, "CDEF:compmout$idef=mout$idef,8,*";
			if ( $defcount == 1) {
				push @argsin, "AREA:compin$idef#$colorin:$i      INPUT";
				push @argsin, "GPRINT:compmin$idef:MAX:%8.3lf %s";
				push @argsin, "GPRINT:compin$idef:AVERAGE:%8.3lf %s";
				push @argsin, "GPRINT:compin$idef:LAST:%8.3lf %s\\r";
				push @argsout, "AREA:compout$idef#$colorout:$i     OUTPUT";
				push @argsout, "GPRINT:compmout$idef:MAX:%8.3lf %s";
				push @argsout, "GPRINT:compout$idef:AVERAGE:%8.3lf %s";
				push @argsout, "GPRINT:compout$idef:LAST:%8.3lf %s\\r";
			}
			else {
				push @argsin, "STACK:compin$idef#$colorin:$i      INPUT";
				push @argsin, "GPRINT:compmin$idef:MAX:%8.3lf %s";
				push @argsin, "GPRINT:compin$idef:AVERAGE:%8.3lf %s";
				push @argsin, "GPRINT:compin$idef:LAST:%8.3lf %s\\r";
				push @argsout, "STACK:compout$idef#$colorout:$i     OUTPUT";
				push @argsout, "GPRINT:compmout$idef:MAX:%8.3lf %s";
				push @argsout, "GPRINT:compout$idef:AVERAGE:%8.3lf %s";
				push @argsout, "GPRINT:compout$idef:LAST:%8.3lf %s\\r";
			}
		}
		$indefs = $indefs.',in'.$idef;
		$outdefs = $outdefs.',out'.$idef;
		$mindefs = $mindefs.',min'.$idef;
		$moutdefs = $moutdefs.',mout'.$idef;
		$math = $math.',+';
	}
	push @args, @argsin, @argsout;
	$indefs =~ (s/^,//); # Removing first comma
	$outdefs =~ (s/^,//); # Removing first comma
	$mindefs =~ (s/^,//); # Removing first comma
	$moutdefs =~ (s/^,//); # Removing first comma
	$math =~ (s/^,\+//); # Removing first comma with plus
	my $maxbits; # Converted bits from bytes
	$maxbits = ($maxbytes*8);
	if ($maxbits > 1000000000) {
		$maxbits = ($maxbits/1000000000).' Gbps';
	}
	elsif ($maxbits > 1000000) {
		$maxbits = ($maxbits/1000000).' Mbps';
	}
	elsif ($maxbits > 1000) {
		$maxbits = ($maxbits/1000).' Kbps';
	}
	push @args,
	"CDEF:fin=$indefs$math,8,*", # IN
	"CDEF:fout=$outdefs$math,8,*", # OUT
	"CDEF:pin=fin,$maxbytes,/,100,*,8,/", # Percentage IN
	"CDEF:pout=fout,$maxbytes,/,100,*,8,/", # Percentage OUT
	"CDEF:fmin=$mindefs$math,8,*", # Max IN
	"CDEF:fmout=$moutdefs$math,8,*", # Max OUT
	"CDEF:pmin=fmin,$maxbytes,/,100,*,8,/", # Percentage max IN
	"CDEF:pmout=fmout,$maxbytes,/,100,*,8,/"; # Percentage max OUT
	if ($type eq "normal") {
		push @args, "AREA:fin#00cc00:Incoming Traffic in Bits per Second      Max speed  ${maxbits}\\l";
		push @args, "LINE1:fout#0000ff:Outgoing Traffic in Bits per Second      Max speed  ${maxbits}\\l";
	}
	elsif ($type eq "neg") {
		push @args, "COMMENT:\\n";
		push @args, "COMMENT:Maximal rate: ${maxbits}\\c";
	}	
	push @args,
	'GPRINT:fmin:MAX:Maximal In\:  %8.3lf %s',
	'GPRINT:pmin:MAX:(%6.2lf%%)',
	'GPRINT:fmout:MAX:Maximal Out\: %8.3lf %s',
	'GPRINT:pmout:MAX:(%6.2lf%%)\l',
	'GPRINT:fin:AVERAGE:Average In\:  %8.3lf %s',
	'GPRINT:pin:AVERAGE:(%6.2lf%%)',
	'GPRINT:fout:AVERAGE:Average Out\: %8.3lf %s',
	'GPRINT:pout:AVERAGE:(%6.2lf%%)\l',
	'GPRINT:fin:LAST:Current In\:  %8.3lf %s',
	'GPRINT:pin:LAST:(%6.2lf%%)',
	'GPRINT:fout:LAST:Current Out\: %8.3lf %s',
	'GPRINT:pout:LAST:(%6.2lf%%)\l';
	RRDs::graph(@args); # Creating graph on filesystem
	if (RRDs::error) {
		my $rrd_error = RRDs::error;
		die "RRDTOOL ERROR: $rrd_error\n";
	}
	# Printing graph from filesystem
	binmode STDOUT;
	open PNG, "</tmp/$name\-$png.png" or die "ERROR: Can't open /tmp/$name\-$png.png";
	print "Content-Type: image/png\n\n";
	my $buf;
	while (sysread PNG, $buf, 8192) {
		print $buf;
	}
	close PNG;
	unlink ("/tmp/$name\-$png.png") or die "ERROR: Can't delete /tmp/$name\-$png.png";
}
#EOF
